/*
 *  Jomar3D, a lobster-running-to-catch game.
 *  Copyright (C) 2006-2008 Jonathan Michalon <studios.chalmion@no-log.org>.
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include <ctype.h>
#include <limits.h>
#include <stdlib.h>
#include <string.h>

#include <GL/glut.h>

#include "core.h"
#include "draw.h"
#include "drawmenu.h"
#include "drawtext.h"
#include "interface.h"
#include "jomar3D.h"
#include "scores.h"
#include "sound.h"
#include "utils.h"


static float        l_cyls[NCYL][4];  /* menu cylinder: 0 angle   1,2,3 rgb color */
static const char*  l_cylstext[NCYL]; /* pointer to the text to draw (NULL if cylinder not to be drawn at all) */

static int l_namesee;            /* name to be shown in the score menu (-1 if none) */

static int l_entries[NMENU] = {40, 5, 1, 1, 8, 1};  /* first: number of text lines before the first menu in the language files
                                                     * then: number of text lines for each menu */

static int l_menunb = 0;         /* number of the menu in text order */

static char* l_soundtxt = NULL;  /* text to print in the sound option cylinder */

/* Color of the menu supports */
static GLfloat l_menuColor[]   = { 0.0, 0.5, 0.7, 1.0 };



/* Init the cylinder's angles */
void
drawmenu_initCyls (void)
{
  int i;
  for (i = 0; i < NCYL; i++)
    l_cyls[i][0] = 0.;
}

/* Free what should */
void
drawmenu_quit (void)
{
  free (l_soundtxt);
}

/* change the color of one cylinder and then draw it */
void
drawmenu_drawHitCylender (int cylnb)
{
  int col;

  do
  {
    col = MYRAND * (3);
  } while (col == 3); /* to have a good proba on 0,1,2 it must be few 3, skip */
  
  l_cyls[cylnb][col + 1] = MYRAND;
  drawmenu_createLists (cylnb);

  drawmenu_redraw ();

  /* reset to normal color */
  l_cyls[cylnb][1] = l_cyls[cylnb][2] = l_cyls[cylnb][3] = 0.5;
  drawmenu_createLists (cylnb);
}

/* print the text of the scores from the menu */
static void
drawmenu_drawScores (void)
{
  char* buf;
  char* buft;
  char* bufs;
  int i, j, pos, place, length;
  int tmpval, toprint;
  Player *playerdrawn;

  playerdrawn = scores_getPlayer (l_namesee);
  if (!playerdrawn)
  {
    fprintf (stderr, "Invalid player ID %d", l_namesee);
    return;
  }

  glColor4f (0.0, 0.2, 0.2, 1.0);

  /* Print the player's name and game count */
  buf = malloc (strlen (playerdrawn->name) + strlen (core_getTextPointer(32)) + 1 +
                intlen (playerdrawn->games, 10) + 1);
  if (buf)
  {
    sprintf (buf, "%s%s %d", playerdrawn->name, core_getTextPointer(32),
             playerdrawn->games);
    drawtext_drawString (750., 650., 0.5, 0.5, TEXT_CENTER, 1400., FALSE, FALSE,
                         buf);
    free (buf);
  }
  else
    perror ("failed to allocate memory");

  /* Calculate the length needed for the lines of scores */
  for (i=0, length=0; i<NSTORE; i++)
  {
    if (playerdrawn->times[i] == 0.)
      continue;
    /* If it's not the firt entry add the '; ' length */
    if (i != 0)
      length += 2;
    /* Now add the length of the longest one the two values */
    length += max (intlen((int)playerdrawn->times[i]/1000., 10) + 3,
                   intlen (playerdrawn->steps[i], 10));
  }
  /* And finaly add the '\0' length */
  length++;
  /* Malloc the two lines */
  buft = malloc (length);
  bufs = malloc (length);
  
  pos = toprint = 0;
  /* Loop on each entry */
  for (i = 0; i < NSTORE; i++)
  {
    /* Skip those from games not done */
    if (playerdrawn->times[i] == 0)
      continue;
    /* Find the smallest not drawn */
    for (j = 0, tmpval = INT_MAX, place = 0; j < NSTORE; j++)
    {
      if (playerdrawn->times[j] == 0)
        continue;
      if (playerdrawn->times[j] < tmpval && playerdrawn->times[j] > toprint)
      {
        tmpval = playerdrawn->times[j];
        place = j;
      }
    }
    toprint = tmpval;
    /* If not first entry add '; ' to both lines */
    if (i != 0)
    {
      sprintf (buft+pos, "; ");
      sprintf (bufs+pos, "; ");
      pos += 2;
    }
    /* Print the values */
    length = max (intlen((int)toprint/1000., 10) + 3,
                  intlen (playerdrawn->steps[place], 10));
    sprintf (buft+pos, "%*.2f", length-3, toprint / 1000.);
    sprintf (bufs+pos, "%*i", length, playerdrawn->steps[place]);
    pos += length;
  }

  /* Draw all that stuff */
  drawtext_drawString (750., 500., 0.4, 0.3, TEXT_CENTER, 1400., FALSE, FALSE,
                       core_getTextPointer(17));
  drawtext_drawString (750., 400., 0.5, 0.3, TEXT_CENTER, 1400., FALSE, FALSE,
                       buft);
  drawtext_drawString (750., 300., 0.4, 0.3, TEXT_CENTER, 1400., FALSE, FALSE,
                       core_getTextPointer(18));
  drawtext_drawString (750., 200., 0.5, 0.3, TEXT_CENTER, 1400., FALSE, FALSE,
                       bufs);

  /* And forget that wonderful work */
  free (buft);
  free (bufs);
}

/* draw the rule menu */
static void
drawmenu_drawRules (void)
{
  char *buf;

  glColor4f (0.0, 0.2, 0.2, 1.0);
  drawtext_drawString (750., 700., 0.4, 0.3, TEXT_CENTER, 1400., FALSE, FALSE,
                       core_getTextPointer(6));
  drawtext_drawString (40., 630., 0.4, 0.30, TEXT_LEFT, 1400., FALSE, FALSE,
                       core_getTextPointer(7));
  drawtext_drawString (60., 580., 0.4, 0.30, TEXT_LEFT, 1400., FALSE, FALSE,
                       core_getTextPointer(8));
  drawtext_drawString (40., 510., 0.4, 0.30, TEXT_LEFT, 1400., FALSE, FALSE,
                       core_getTextPointer(9));
  drawtext_drawString (60., 460., 0.4, 0.30, TEXT_LEFT, 1400., TRUE, FALSE,
                       core_getTextPointer(10));
  drawtext_drawString (60., 410., 0.4, 0.30, TEXT_LEFT, 1400., TRUE, FALSE,
                       core_getTextPointer(11));
  drawtext_drawString (60., 360., 0.4, 0.30, TEXT_LEFT, 1400., TRUE, FALSE,
                       core_getTextPointer(12));
  
  buf = malloc (uintlen (ULONG_MAX, 10) + 1 + strlen (core_getTextPointer(13)) +1);
  if (buf)
  {
    sprintf (buf, "%lu %s", ULONG_MAX, core_getTextPointer(13));
    drawtext_drawString (60., 310., 0.4, 0.30, TEXT_LEFT, 1400., TRUE, FALSE,
                         buf);
  }
  else
    perror ("failed to allocate memory");
  free (buf);

  drawtext_drawString (40., 240., 0.4, 0.30, TEXT_LEFT, 1400., FALSE, FALSE,
                       core_getTextPointer(14));
  drawtext_drawString (60., 190., 0.4, 0.30, TEXT_LEFT, 1400., TRUE, FALSE,
                       core_getTextPointer(15));
  drawtext_drawString (60., 140., 0.4, 0.30, TEXT_LEFT, 1400., TRUE, FALSE,
                       core_getTextPointer(16));
}

/* draw the finds menu */
static void
drawmenu_drawFinds (void)
{
  glColor4f (0.0, 0.2, 0.2, 1.0);
  drawtext_drawString (40., 700., 0.3, 0.30, TEXT_LEFT, 1400., FALSE, FALSE,
                       core_getTextPointer(19));
  drawtext_drawString (40., 650., 0.3, 0.30, TEXT_LEFT, 1400., FALSE, FALSE,
                       core_getTextPointer(20));
  drawtext_drawString (40., 580., 0.3, 0.30, TEXT_LEFT, 1400., FALSE, FALSE,
                       core_getTextPointer(21));
  drawtext_drawString (40., 530., 0.3, 0.30, TEXT_LEFT, 1400., FALSE, FALSE,
                       core_getTextPointer(22));
  drawtext_drawString (40., 460., 0.3, 0.30, TEXT_LEFT, 1400., FALSE, FALSE,
                       core_getTextPointer(23));
  drawtext_drawString (40., 390., 0.3, 0.30, TEXT_LEFT, 1400., FALSE, FALSE,
                       core_getTextPointer(24));
  drawtext_drawString (40., 340., 0.3, 0.30, TEXT_LEFT, 1400., FALSE, FALSE,
                       core_getTextPointer(25));
  drawtext_drawString (40., 270., 0.3, 0.30, TEXT_LEFT, 1400., FALSE, FALSE,
                       core_getTextPointer(26));
  drawtext_drawString (40., 200., 0.3, 0.30, TEXT_LEFT, 1400., FALSE, FALSE,
                       core_getTextPointer(27));
  drawtext_drawString (750., 100., 0.4, 0.30, TEXT_CENTER, 1400., TRUE, FALSE,
                       core_getTextPointer(28));
}

/* draw the right text for the current menu */
void
drawmenu_draw (void)
{
  drawtext_begin ();

  glMatrixMode (GL_PROJECTION);
  glPushMatrix ();
  glLoadIdentity ();
  gluOrtho2D (0, 1500, 0, 1500);


  glColor4f (1.0, 1.0, 1.0, 0.8);
  if ((core_checkState (JOMAR3D_MENU_DISPSCORE) && l_namesee != -1) ||
      !core_checkState (JOMAR3D_MENU_DISPSCORE))
    glRecti (20, 20, 1480, 750);

  glLineWidth (1.6);
  if (core_checkState (JOMAR3D_MENU_RULES))
    drawmenu_drawRules ();
  if (core_checkState (JOMAR3D_MENU_DISPSCORE) && l_namesee != -1)
    drawmenu_drawScores ();
  if (core_checkState (JOMAR3D_MENU_FINDS))
    drawmenu_drawFinds ();

  glMatrixMode (GL_PROJECTION);
  glPopMatrix ();

  drawtext_end ();

  /*glPopAttrib ();*/
  glMatrixMode (GL_MODELVIEW);
}

/* create the needed lists for the menu */
void
drawmenu_createLists (int listnb /* list number to do (0 = all) */)
{                               
  int i;
  float cylssize = 60., cylcol[4];
  static float menuwidth;       /* Width of each text cylinder for the current menu. */
  int firsttext = 0;            /* Position of the first text of this menu. */

  if (!core_checkState (JOMAR3D_INMENU))
    return;

  i = listnb;
  if (listnb != 0)
    goto remake;

/* cylinders initialization */
  for (i = 0; i < NCYL; i++)
  {
    l_cyls[i][1] = l_cyls[i][2] = l_cyls[i][3] = 0.5;
    l_cylstext[i] = NULL;
  }
  for (i = 0; i < l_menunb; i++)
    firsttext += l_entries[i];

  debugmsg (("Menu first text pos: %d, is %s\n", firsttext,
            core_getTextPointer(firsttext)));

  if (!core_checkState (JOMAR3D_MENU_OPTION))
    for (i = 0; i < l_entries[l_menunb]; i++)
      l_cylstext[i] = core_getTextPointer(firsttext + i);

  if (core_checkState (JOMAR3D_MENU_OPTION))
  {
    l_cylstext[0] = core_getLanguageName ();
    l_cylstext[1] = core_getSkinName ();
    l_cylstext[2] = NULL;
#if (USESOUND)
    if (core_checkState(JOMAR3D_PLAYMUSIC))
      l_soundtxt = mystrdup (MUSICDIR "/*.ogg, *.wav, *.voc & sounds");
    else
      l_soundtxt = mystrdup ("X");
    if (l_soundtxt && core_checkState (JOMAR3D_CANMUSIC))
      l_cylstext[2] = l_soundtxt;
    else
      free (l_soundtxt);
#endif /* (USESOUND) */
    l_cylstext[3] = core_getTextPointer(firsttext + 4 + draw_getQuality ());
    l_cylstext[4] = core_getTextPointer(firsttext + 7);
  }
  if (core_checkState (JOMAR3D_MENU_DISPSCORE))
  {
    for (i = 0; i < MAXNAMES; i++)
    {
      l_cylstext[i] = scores_getName(i);
      if (l_cylstext[i] == NULL)
        break;
    }
    l_cylstext[i] = core_getTextPointer(firsttext);
  }
  for (i = 0, menuwidth = 0.; i < NCYL - 1; i++)
    if (l_cylstext[i] != NULL)
      menuwidth = max (drawtext_stringWidth (l_cylstext[i]), menuwidth);
  menuwidth *= 1.1;             /* be a little brighter than exact width */
  if (menuwidth > 2500)
    menuwidth = 2500;
  
  debugmsg (("Menu width: %f\n", menuwidth));


/* create the list to call to draw the current menu */
  glNewList (MENU, GL_COMPILE);
  glShadeModel (GL_SMOOTH);
  glPushMatrix ();
  glTranslatef (0., 0., -1100.);
  glScalef (1500./menuwidth, 1., 1.);
  glTranslatef (0., NCYL * cylssize * 1.1, 0.);
  glPushMatrix ();
  glMaterialfv (GL_FRONT, GL_DIFFUSE, l_menuColor);
  glRotatef (90, 1, 0, 0);
  glTranslatef (menuwidth / 2. + cylssize * 1.1, 0., 0.);
  gluCylinder (draw_getQuadric (), cylssize, cylssize, (NCYL + 1) * cylssize * 2.6,
               5*draw_getVertMult (), 1);
  glTranslatef (-menuwidth - cylssize * 1.1 * 2., 0., 0.);
  gluCylinder (draw_getQuadric (), cylssize, cylssize, (NCYL + 1) * cylssize * 2.6,
               5*draw_getVertMult (), 1);
  glPopMatrix ();
  glPushMatrix ();
  glLineWidth (2.1);
  glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_BLEND);
  for (i = 1; i <= NCYL; i++)
  {
    glTranslatef (0., -cylssize * 2.5, 0.);
    if (l_cylstext[i - 1] != NULL)
      glCallList (MENU + i);
  }
  glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
  glPopMatrix ();
  if (core_checkState (JOMAR3D_MENU_OPTION))
  {
    glLineWidth (1.5);
    drawtext_begin ();
    glTranslatef (-menuwidth / 1.7, cylssize, 0.);
    for (i = 0; i<4; i++)
    {
      glTranslatef (0., -cylssize * 2.6, 0.);
      glColor4f ( 0.9, 0.9, 0.8, 1.);
      glRecti (-300, -20, 300, 70);
      glColor4f ( 0., 0., 0., 1.);
      glBegin (GL_LINE_LOOP);
        glVertex2f (-300, -20);
        glVertex2f (-300,  70);
        glVertex2f ( 300,  70);
        glVertex2f ( 300, -20);
      glEnd ();
      glColor4f ( .1, .1, .6, 1.);
      drawtext_drawString (0., 0., 0.5, 0.4, TEXT_CENTER, 2300., FALSE, FALSE,
                           core_getTextPointer(firsttext + i));
    }
    drawtext_end ();
  }
  glPopMatrix ();
  glShadeModel (GL_FLAT);
  glEndList ();

/* set the loop indice for the creation of all the cylinders lists */
  i = 1;

remake:;
  if (l_cylstext[i - 1] == NULL)
    return;
/* create the cylinder list */
  glNewList (MENU + i, GL_COMPILE);
  glPushMatrix ();
  glTranslatef (menuwidth / 2., 0., 0.);
  glRotatef (-90, 0, 1, 0);
  glRotatef (-l_cyls[i-1][0], 0, 0, 1);
  cylcol[0] = l_cyls[i][1];
  cylcol[1] = l_cyls[i][2];
  cylcol[2] = l_cyls[i][3];
  cylcol[3] = 1.;
  glMaterialfv (GL_FRONT, GL_DIFFUSE, cylcol);
  glEnable (GL_TEXTURE_2D);
  glBindTexture (GL_TEXTURE_2D, draw_getTexID(TEXID_MENU));
  gluQuadricTexture (draw_getQuadric (), GL_TRUE);
  glLoadName (i);
  gluCylinder (draw_getQuadric (), cylssize, cylssize, menuwidth, 6*draw_getVertMult (), 1);
  glLoadName (0);
  gluQuadricTexture (draw_getQuadric (), GL_FALSE);
  glDisable (GL_TEXTURE_2D);
  glPopMatrix (); 
  glColor4f (0.0, 0.0, 0.0, 1.);
  if (core_checkState (JOMAR3D_MENU_OPTION) && i != 5)
  {
    glDisable (GL_LIGHTING);
    glBegin (GL_LINE_STRIP);
      glVertex3f (-30., 50., 0.);
      glVertex3f (0., 60., 0.);
      glVertex3f (30., 50., 0.);
    glEnd ();
    glBegin (GL_LINE_STRIP);
      glVertex3f (-30., -50., 0.);
      glVertex3f (0., -60., 0.);
      glVertex3f (30., -50., 0.);
    glEnd ();
    glEnable (GL_LIGHTING);
  }
  if (abs(l_cyls[i-1][0]) < 80. || abs(l_cyls[i-1][0]) > 280. )
  {
    glPushMatrix ();
    glRotatef (l_cyls[i - 1][0], 1, 0, 0);
    glTranslatef (0., -cylssize * .5, cylssize * 0.7);
    drawtext_begin ();
    drawtext_drawString (0., 0., 1.0, 0.4, TEXT_CENTER, 2300., FALSE, FALSE,
                         l_cylstext[i - 1]);
    drawtext_end ();
    glPopMatrix ();
  }
  glEndList ();

/* if we are initializing the whole menu, loop on each cylinder */
  next:
  if (listnb == 0 && i != NCYL)
  {
    i++;
    if (l_cylstext[i - 1] == NULL)
      goto next;
    goto remake;
  }
}

/* redraw the menu WITHOUT the background (lobster etc.) */
void
drawmenu_redraw (void)
{
  if (!draw_postIntro ())
    return;

  glDepthFunc (GL_ALWAYS);
  glDepthMask (GL_FALSE);
  if (core_checkState (JOMAR3D_MENU_RULES) ||
      core_checkState (JOMAR3D_MENU_FINDS) ||
      core_checkState (JOMAR3D_MENU_DISPSCORE))
    {
      GLdouble planeEqn[] = { 0.000, 1.000, 0.0, 0.0 };  /* clipping plane equation */
      
      glClipPlane (GL_CLIP_PLANE0, planeEqn);   /* define clipping  plane */
      glEnable (GL_CLIP_PLANE0); /* and enable it */
    }
  glCallList (MENU);
  glutSwapBuffers ();
  glDisable (GL_CLIP_PLANE0);
  glDepthMask (GL_TRUE);
  glDepthFunc (GL_LESS);
}

/* Set the current menu number */
void
drawmenu_setMenuNumber (int number)
{
  l_menunb = number;
}

/* Set the name number to see in the scores */
void
drawmenu_setNameToSee (int number)
{
  l_namesee = number;
}

/* Set the angle of a cylinder */
void
drawmenu_setCylinderAngle (int cylnb, float angle)
{
  l_cyls[cylnb][0] = angle;
  debugmsg (("angle of cylinder %i now %f\n", cylnb, l_cyls[cylnb][0]));
}

/* Return the fisrt letter of the given cylinder */
char
drawmenu_getCylinderLetter (int cylnb)
{
  if (l_cylstext[cylnb] == NULL)
    return 0;

  return tolower (*l_cylstext[cylnb]);
}
